/*
 *
 *
 *    cavg.h
 *
 *    Integrator template.
 *
 *    Copyright (C) 2007-2016 by Matt Roberts.
 *    License: GNU GPL2 (www.gnu.org)
 *
 *
 */

#ifndef __CAVG_H
#define __CAVG_H

#ifdef __linux__
#include <cstdlib>
#endif

/*
 *
 *   class Averager<T>
 *
 */
template <typename sample_t, typename accum_t = sample_t>
class Averager {
	private:
		// the length of the integrator
		size_t length;
		// the list of samples
		sample_t * samples;
		// pointer for walking through the list of samples
		sample_t * ptr;
		// the accumulator
		accum_t sum;

	public:
		Averager(void) : length(0), samples(0), ptr(0), sum(0){
			// nop
		}

		Averager(size_t length) : length(0), samples(0), ptr(0), sum(0) {
			resize(length);
		}
	private: // disallowed ctors
		Averager(const Averager&);

	public:
		size_t size(void) const { return length; };
		accum_t accum(void) const { return sum; };
		accum_t value(void) const { return sum / static_cast<accum_t>(length); };
		operator accum_t() const { return value(); };

	public:
		void resize(size_t n);
		void add(const sample_t &s);
};

template <typename sample_t, typename accum_t>
inline void Averager<sample_t, accum_t>::resize(size_t n) {
	// resize the array
	if (samples) {
		delete[] samples;
	}
	length = n;
	samples = new sample_t[length];

	// initialize the samples
	for (ptr = samples; ptr != (samples + length); ++ptr) {
		*ptr = 0;
	}

	// zero out the sum
	sum = 0;

	// the i/o pointer starts at the first element
	ptr = samples;
}

template <typename sample_t, typename accum_t>
inline void Averager<sample_t, accum_t>::add(const sample_t &s) {
	// subtract the value at the current i/o element
	sum -= *ptr;
	// add the new value to the sum
	sum += s;
	// store the new value in the current i/o element
	*ptr = s;
	// advance to the next element
	++ptr;

	// wrap the i/o pointer if needed
	if (ptr == (samples + length)) {
		ptr = samples;
	}
}

#endif
